/*
  LED bar graph: Turns on a series of LEDs based on the value of an analog sensor.
  
  This is a simple way to make a bar graph display. Though this graph uses 10 LEDs, you can use any number 
  by changing the LED count and the pins in the array.
  This method can be used to control any series of digital outputs that depends on an analog input.
  
  CIRCUIT: 10 LEDs from pins 2 through 11. Anode (+) side of each LED is attached to the Arduino pins. 
  The cathode (-) side of each LED is attached to a current limiting resistor (220/330/470 ohm) to ground.
  A 3-pin potentiometer (pot, for short) is attached such that the left pin goes to Vcc, the right pin goes 
  to ground, and the middle pin goes to pin A0. The pot adjustment screw or knob is turned from one extreme 
  to the other, increasing or decreasing the voltage level going to analog pin, A0. The result should be an 
  increase in the number of LEDs being illuminated or turned off, depending on the direction turned.
  
  LEARNINGS: In addition to learning how to use a 'for' loop to quickly assign the pins as outputs, we'll also learn 
  how to use the 'map' command. Analog pins like A0 through A5 use an ADC with a range of 0 to 1023. As the potentiometer
  is rotated from one extreme (fully off, 0V) to the other (fully on, 5V), we want to be able to show the rotation level by
  turning LEDs on or off. 0v = all LEDs off, 1/2 max voltage = half the LEDs on, max voltage = all LEDs on.
  The syntax parameters are 'y map(x,fromLow,fromHigh,toLow,toHigh);'
  If there are 10 LEDs and we're using 5v with an Arduino UNO, then 10/5 = 500mv divisions. For each increase in voltage 
  by 500mV, we want another LED illuminated. At just 500mV, one LED is on. At 4500mV (4.5V), nine LEDs are on.
  The map command re-maps a number from one range to another. It requires 4 values such as 'fromLow', 'toLow', 'fromHigh', and 'toHigh'. 
  Value 'fromLow' would be 0, as would be 'toLow'. Value 'fromHigh' would be 1023 (remember the analog to digital converter ADC range 
  is 0-1023) and 'toHigh' would be 10 (the range is 0 LEDs to 10 LEDs).
  An example would be 'x = map( reading, 0 ADC, 1023 ADC, 0 LEDs, 10 LEDs);' but with just the numbers.
  The program runs continously reading analog pin A0 for a change in voltage. At 0v, all the LEDs are off. As the pot adjuster is turned so the voltage
  is higher, then 500mV will map to the first LED, 1, and we expect it to be illuminated. All other LEDs are sent a LOW which turns them off.
  As more voltage is added via the pot, more LEDs in sequence will be turned on; all others numerically higher than 'thisLed' will be turned off.
  
  TROUBLESHOOTING: To better understand what is happening, startup the Serial Monitor with a baud rate of 9600. The function 'serMon', short for 'serialMonitor', 
  has been added to the sketch to show the ADC value and equivalent computed voltage on pin A0, the current LED being serviced, and the maximum LED level 
  being driven to a high value to illuminate it.
    
  Created 4 Sep 2010 by Tom Igoe. 'serMon' addition 2018-01-17 by dp.
  
  The example code is in the public domain. http://www.arduino.cc/en/Tutorial/BarGraph
*/

// These constants won't change:
const int analogPin = A0;                          // The Arduino pin to which the potentiometer middle pin is attached.
const int ledCount = 10;                           // The number of LEDs in the bar graph.

int ledPins[] = {2, 3, 4, 5, 6, 7, 8, 9, 10, 11};  // An array of pin numbers to which LEDs are attached.
float sensorReading = 0;                           // Used by serMon to get the computed voltage level being read on pin A0; 'int' only show whole integer values.
int thisLed = 0;                                   // The current LED being serviced.
int ledLevel = 0;                                  // A number between 0 and 10 that maps the illuminated LED count to the sensor reading.
///////////////////////////////////////
void setup() {                                             // This portion runs once upon reset.
  Serial.begin(9600);                                      // We've added the Serial Monitor to show the actual voltage and ADC values. Use the magnifying glass button in the top right corner.
  for (int thisLed = 0; thisLed < ledCount; thisLed++) {   // Loop through the pin array and set all the pins to output.
    pinMode(ledPins[thisLed], OUTPUT);                     // ledPin 2, ledPin 3...ledPin 11.
  }                                                        // This method saves a lot of code lines of 'pinMode(pin#, OUTPUT)'.
}
///////////////////////////////////////
void loop() {
  sensorReading = analogRead(analogPin);                // Read the potentiometer value (0 to 1023) attached to A0.
  ledLevel = map(sensorReading, 0, 1023, 0, ledCount);  // Map the result to a range from 0 to the number of LEDs: sensorReading, fromLow, fromHigh, fromLow, fromHigh.
  for (thisLed = 0; thisLed < ledCount; thisLed++) {    // Loop through the LED array.
    serMon();                                           // Optional function added to show the current variable values in the Serial Monitor.
    if (thisLed < ledLevel) {                           // If the array element's index is less than ledLevel, turn the pin on for this element.
      digitalWrite(ledPins[thisLed], HIGH);
    }
    else {                                              // Turn off all Arduino pins higher than the ledLevel.
      digitalWrite(ledPins[thisLed], LOW);
    }
  }
}
///////////////////////////////////////
void serMon() {                                         // An optional function that can be used for troubleshooting.
    Serial.print("sensorReading: ");                    // Display the sensorReading value between 0 and 1023.              
    Serial.print(sensorReading);
    Serial.print("\t");                                 // Move the next column to be displayed over by one tab.
    Serial.print("Volts: ");                            // Displays the computed (unmeasured) voltage level being read by pin A0.
    Serial.print(sensorReading / 204.6, 2);             // 1023 scale value / 5.0v = 204.6mv. '2' indicates 2 decimal places of precision.
    Serial.print("\t"); 
    Serial.print("thisLED: ");
    Serial.print(thisLed);                              // Displays the current LED number being displayed, from 0 to 10.
    Serial.print("\t"); 
    Serial.print("ledLevel: ");                         // Displays mapped LED level being displayed: '0 to 1023' maps '0 to 10'. 
    Serial.println(ledLevel); 
    delay(250);                                         // Display at a reasonable rate.
}

